{*****************************************************}
{                                                     }
{ Customize dialog                                    }
{                                                     }
{ Copyright(c) 2001 Soford - Software For Developers  }
{ http://www.soford.com                               }
{ e-mail: contact@soford.com                          }
{                                                     }
{*****************************************************}

unit EasyCustomDlg;

interface
{$I Easy.inc}
uses
  {$IFNDEF EASY_CLX}
  Windows, Messages, Graphics, Controls, Forms, Dialogs, ComCtrls, ExtCtrls, StdCtrls, Buttons, Menus,
  {$ELSE}
  QForms, QControls, QGraphics, QStdCtrls, QExtCtrls, QComCtrls, QDialogs, QButtons,
  {$ENDIF}
  SysUtils, Classes,

  EasyBox, EasyClasses, EasyParser, EasyEditor, EasyEditSource, EasyUtils, EasyEditorSettings, EasyControls;


type
  TCustomizeForm = class(TForm)
    TreeViewOptions: TTreeView;
    ColorDialog1: TColorDialog;
    HolderPanel: TPanel;
    PageControl: TPageControl;
    EditorSheet: TTabSheet;
    DisplaySheet: TTabSheet;
    DispPanel: TPanel;
    gbMarginAndGutter: TGroupBox;
    lbRightMargin: TLabel;
    lbGutterWidth: TLabel;
    chbVisibleRightMargin: TCheckBox;
    chbVisibleGutter: TCheckBox;
    ColorsSheet: TTabSheet;
    ColorsPanel: TPanel;
    plButtons: TPanel;
    btOK: TButton;
    btCancel: TButton;
    btHelp: TButton;
    EdPanel: TPanel;
    gbEditorOptions: TGroupBox;
    cbAutoIndentMode: TCheckBox;
    cbInsertMode: TCheckBox;
    cbUseTabCharacter: TCheckBox;
    cbSmartTab: TCheckBox;
    cbOptimalFill: TCheckBox;
    cbBackspaceUnindents: TCheckBox;
    cbCursorThroughTabs: TCheckBox;
    cbGroupUndo: TCheckBox;
    cbCursorBeyondEOF: TCheckBox;
    cbKeepTrailingBlanks: TCheckBox;
    cbPersistentBlocks: TCheckBox;
    cbOverwriteBlocks: TCheckBox;
    cbDoubleClickLine: TCheckBox;
    cbFindTextAtCursor: TCheckBox;
    cbForceCutAndCopyEnabled: TCheckBox;
    cbUseSyntaxHighlight: TCheckBox;
    cbEnableSelection: TCheckBox;
    cbEnableDragging: TCheckBox;
    cbCursorBeyondEol: TCheckBox;
    cbEnableSearchHighlight: TCheckBox;
    lbKeyMapping: TLabel;
    cbEditorSpeedSetting: TComboBox;
    lbBlockIndent: TLabel;
    lbTabStops: TLabel;
    gbDispOptions: TGroupBox;
    lbEditorFont: TLabel;
    lbSize: TLabel;
    cbEditorFont: TComboBox;
    cbSize: TComboBox;
    pnAaBbYyZz: TPanel;
    lbSample: TLabel;
    GroupBox1: TGroupBox;
    cbColorSpeedSetting: TComboBox;
    EasycbForegroundColor: TEasyColorBox;
    ForeColorBtn: TBitBtn;
    gbTextAttributes: TGroupBox;
    chbBold: TCheckBox;
    chbItalic: TCheckBox;
    chbUnderline: TCheckBox;
    EasycbBackgroundColor: TEasyColorBox;
    BackColorBtn: TBitBtn;
    lsbElements: TListBox;
    gbUseDefault: TGroupBox;
    chbForeground: TCheckBox;
    chbBackground: TCheckBox;
    btOpen: TButton;
    btSave: TButton;
    cbRightMargin: TEasyHistoryEditor;
    cbGutterWidth: TEasyHistoryEditor;
    cbTabStops: TEasyHistoryEditor;
    cbBlockIndent: TEasyHistoryEditor;
    procedure FormCreate(Sender: TObject);
    procedure TreeViewOptionsChange(Sender: TObject; Node: TTreeNode);
    procedure lsbElementsClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cbColorSpeedSettingChange(Sender: TObject);
    procedure EasycbForegroundColorChange(Sender: TObject);
    procedure EasycbBackgroundColorChange(Sender: TObject);
    procedure chbBoldClick(Sender: TObject);
    procedure chbItalicClick(Sender: TObject);
    procedure chbUnderlineClick(Sender: TObject);
    procedure chbForegroundClick(Sender: TObject);
    procedure chbBackgroundClick(Sender: TObject);
    procedure ForeColorBtnClick(Sender: TObject);
    procedure BackColorBtnClick(Sender: TObject);
    procedure cbEditorFontChange(Sender: TObject);
    procedure cbSizeChange(Sender: TObject);
  private
    FStyles : TEasyParserStyles;
    procedure ShowPanel(Panel : TPanel);
    procedure HidePanel(Panel : TPanel);
    { Private declarations }
  public
    procedure ShowOptionsPanel;
    procedure SetBackAndForeStyles(Value : integer);
    procedure SaveSettings;
    procedure LoadSettings;
    { Public declarations }
  end;

var
  CustomizeForm : TCustomizeForm;


procedure ShowEditorSettings;

implementation

{$IFNDEF EASY_CLX}
{$R *.DFM}
{$ELSE}
{$R *.xfm}
{$ENDIF}

{---------------------------------------------------}

procedure ShowEditorSettings;
begin
  with TCustomizeForm.Create(Application) do
  try
    if ShowModal = mrOk then
      SaveSettings;
  finally
    Free;
  end;
end;

{---------------------------------------------------}

procedure TCustomizeForm.FormCreate(Sender: TObject);
var
  i : integer;
  F : TFont;
  s : string;
begin
  {$IFDEF _DELPHI4}
  TreeViewOptions.Color := clBtnFace;
  {$ENDIF}
  FStyles := TEasyParserStyles.Create(TEasyParserStyle);
  FStyles.Clear;
  for i := Low(ctDefault) to High(ctDefault) do
  begin
    with TEasyParserStyle(FStyles.Add) do
      Name := ctDefault[i].Name;
    lsbElements.Items.Add(ctDefault[i].Desc);
  end;
  s := AddSlash(GetTempDir) + 'dialogs.ini';
  cbBlockIndent.FileName := s;
  cbTabStops.FileName := s;
  cbRightMargin.FileName := s;
  cbGutterWidth.FileName := s;
  cbBlockIndent.LoadHistory;
  cbTabStops.LoadHistory;
  cbRightMargin.LoadHistory;
  cbGutterWidth.LoadHistory;

  LoadSettings;

  lsbElements.ItemIndex := 0;
  SetBackAndForeStyles(0);
  cbEditorFontChange(Self);

  Height := Height - (PageControl.Height - EditorSheet.Height);
  ShowPanel(EdPanel);
  TreeViewOptions.FullExpand;
  TreeViewOptions.Selected := TreeViewOptions.Items[1];

  F := TFont.Create;
  try
    with Screen.Fonts do
      for i := 0 to Count - 1 do
      begin
        F.Name := Strings[i];
        if IsMonoFont(F) then
          cbEditorFont.Items.Add(F.Name);
      end;
    cbEditorFont.Text := EasyEditSettings.Font.Name;
  finally
    F.Free;
  end;
end;

{---------------------------------------------------}

procedure TCustomizeForm.ShowPanel(Panel : TPanel);
var
  i : integer;
begin
  with Panel do
    for i := ControlCount - 1 downto 0 do
      if Controls[i] is TPanel then
        Controls[i].Hide;
  Panel.Parent := HolderPanel;
  Panel.Visible := true;
  Panel.BringToFront;
end;

{---------------------------------------------------}

procedure TCustomizeForm.HidePanel(Panel : TPanel);
begin
  Panel.Visible := false;
end;

{---------------------------------------------------}

procedure TCustomizeForm.ShowOptionsPanel;
begin
  with TreeViewOptions do
    if Selected <> nil then
      case Selected.AbsoluteIndex of
        1 :
        begin
          ShowPanel(EdPanel);
          HidePanel(DispPanel);
          HidePanel(ColorsPanel);
        end;
        2 :
        begin
          ShowPanel(DispPanel);
          HidePanel(EdPanel);
          HidePanel(ColorsPanel);
        end;
        3 :
        begin
          ShowPanel(ColorsPanel);
          HidePanel(EdPanel);
          HidePanel(DispPanel);
        end;
      end;
end;

{---------------------------------------------------}

procedure TCustomizeForm.TreeViewOptionsChange(Sender: TObject; Node: TTreeNode);
begin
  ShowOptionsPanel;
end;

{---------------------------------------------------}

procedure TCustomizeForm.SetBackAndForeStyles(Value : integer);
begin
  if Value >= 0 then
    with FStyles.Items[Value] do
    begin
      EasycbForegroundColor.SelectedColor := FontColor;
      EasycbBackgroundColor.SelectedColor := Color;
      chbBold.Checked := fsBold in FontStyle;
      chbItalic.Checked := fsItalic in FontStyle;
      chbUnderline.Checked := fsUnderline in FontStyle;
    end;
end;

{-------------------------------------------------}

procedure TCustomizeForm.chbItalicClick(Sender: TObject);
begin
  with FStyles.Items[lsbElements.ItemIndex] do
    if chbItalic.Checked then
      FontStyle := FontStyle + [fsItalic]
    else
      FontStyle := FontStyle - [fsItalic];
end;

{-------------------------------------------------}

procedure TCustomizeForm.chbUnderlineClick(Sender: TObject);
begin
  with FStyles.Items[lsbElements.ItemIndex] do
    if chbUnderline.Checked then
      FontStyle := FontStyle + [fsUnderline]
    else
      FontStyle := FontStyle - [fsUnderline];
end;

{-------------------------------------------------}

procedure TCustomizeForm.chbForegroundClick(Sender: TObject);
begin
//
end;

{-------------------------------------------------}

procedure TCustomizeForm.chbBackgroundClick(Sender: TObject);
begin
//
end;

{-------------------------------------------------}

procedure TCustomizeForm.ForeColorBtnClick(Sender: TObject);
begin
  with EasycbForegroundColor, ColorDialog1 do
  begin
    Color := ColorToRGB(SelectedColor);
    if Execute then
      CustomColor := Color;
  end;
end;

{-------------------------------------------------}

procedure TCustomizeForm.BackColorBtnClick(Sender: TObject);
begin
  with EasycbBackgroundColor, ColorDialog1 do
  begin
    Color := ColorToRGB(SelectedColor);
    if Execute then
      CustomColor := Color;
  end;
end;

{-------------------------------------------------}

procedure TCustomizeForm.lsbElementsClick(Sender: TObject);
begin
  SetBackAndForeStyles(lsbElements.ItemIndex);
end;

{-------------------------------------------------}

procedure TCustomizeForm.FormDestroy(Sender: TObject);
begin
  FStyles.Free;
end;

{-------------------------------------------------}

procedure TCustomizeForm.cbColorSpeedSettingChange(Sender: TObject);

  procedure LoadStyle_ (Value : TEasyStylesConstArray);
  var
    i : integer;
  begin
    for i := Low(Value) to High(Value) do
    begin
      with TEasyParserStyle(FStyles.Items[i]) do
      begin
        FontStyle := Value[i].Style;
        FontColor := Value[i].Color;
        Color := Value[i].BkColor;
      end;
    end;
  end;

begin
  case cbColorSpeedSetting.ItemIndex of
    0 : LoadStyle_(ctDefault);
    1 : LoadStyle_(ctClassic);
    2 : LoadStyle_(ctOcean);
    3 : LoadStyle_(ctTwilight);
    else
      LoadStyle_(ctDefault);
  end;
  lsbElementsClick(Self);
end;

{-------------------------------------------------}

procedure TCustomizeForm.EasycbForegroundColorChange(Sender: TObject);
begin
  with FStyles.Items[lsbElements.ItemIndex] do
    FontColor := EasycbForegroundColor.SelectedColor;
end;

{-------------------------------------------------}

procedure TCustomizeForm.EasycbBackgroundColorChange(Sender: TObject);
begin
  with FStyles.Items[lsbElements.ItemIndex] do
    Color := EasycbBackgroundColor.SelectedColor;
end;

{-------------------------------------------------}

procedure TCustomizeForm.chbBoldClick(Sender: TObject);
begin
  with FStyles.Items[lsbElements.ItemIndex] do
    if chbBold.Checked then
      FontStyle := FontStyle + [fsBold]
    else
      FontStyle := FontStyle - [fsBold];
end;

{-------------------------------------------------}

procedure TCustomizeForm.cbEditorFontChange(Sender: TObject);
begin
  with cbEditorFont do
    pnAaBbYyZz.Font.Name := Text;
end;

{-------------------------------------------------}

procedure TCustomizeForm.cbSizeChange(Sender: TObject);
begin
  with cbSize do
    pnAaBbYyZz.Font.Size := StrToInt(Text);
end;

{-------------------------------------------------}

procedure TCustomizeForm.SaveSettings;
begin
  with EasyEditSettings do
  begin
    BeginUpdate;
    try
      if cbAutoIndentMode.Checked then
        SourceOptions := SourceOptions + [srAutoIndent]
      else
        SourceOptions := SourceOptions - [srAutoIndent];
      EditOverwrite := not cbInsertMode.Checked;
      if cbUseTabCharacter.Checked then
        SourceOptions := SourceOptions + [srUseTab]
      else
        SourceOptions := SourceOptions - [srUseTab];
      if cbSmartTab.Checked then
        SourceOptions := SourceOptions + [srSmartTab]
      else
        SourceOptions := SourceOptions - [srSmartTab];
      if cbOptimalFill.Checked then
        SourceOptions := SourceOptions + [srOptimalFill]
      else
        SourceOptions := SourceOptions - [srOptimalFill];
      if cbBackspaceUnindents.Checked then
        SourceOptions := SourceOptions + [srBackUnindents]
      else
        SourceOptions := SourceOptions - [srBackUnindents];
      if cbCursorThroughTabs.Checked then
        SourceOptions := SourceOptions + [srCursorThroughTabs]
      else
        SourceOptions := SourceOptions - [srCursorThroughTabs];
      if cbGroupUndo.Checked then
        SourceOptions := SourceOptions + [srGroupUndo]
      else
        SourceOptions := SourceOptions - [srGroupUndo];
      if cbCursorBeyondEOF.Checked then
        SourceOptions := SourceOptions + [srBeyondEof]
      else
        SourceOptions := SourceOptions - [srBeyondEof];
      if cbCursorBeyondEol.Checked then
        SourceOptions := SourceOptions + [srBeyondEol]
      else
        SourceOptions := SourceOptions - [srBeyondEol];
      if cbPersistentBlocks.Checked then
        Options := Options + [eoPersistentBlocks]
      else
        Options := Options - [eoPersistentBlocks];
      if cbOverwriteBlocks.Checked then
        Options := Options + [eoOverWriteBlocks]
      else
        Options := Options - [eoOverWriteBlocks];
      if cbEnableSelection.Checked then
        Options := Options + [eoEnableSelection]
      else
        Options := Options - [eoEnableSelection];
      if cbEnableDragging.Checked then
        Options := Options + [eoEnableDragging]
      else
        Options := Options - [eoEnableDragging];
      if cbEnableSearchHighlight.Checked then
        Options := Options + [eoEnableSearchHighlight]
      else
        Options := Options - [eoEnableSearchHighlight];
      if cbDoubleClickLine.Checked then
        Options := Options + [eoDblClickLine]
      else
        Options := Options -  [eoDblClickLine];
      if cbFindTextAtCursor.Checked then
        Options := Options + [eoForceCutCopy]
      else
        Options := Options - [eoForceCutCopy];
      if chbVisibleRightMargin.Checked then
        Options := Options + [eoPaintMargin]
      else
        Options := Options - [eoPaintMargin];

      if chbVisibleGutter.Checked then
        Options := Options + [eoPaintGutter]
      else
        Options := Options - [eoPaintGutter];

      if cbKeepTrailingBlanks.Checked then
        SourceOptions := SourceOptions + [srLeaveTrailSpaces]
      else
        SourceOptions := SourceOptions - [srLeaveTrailSpaces];

      ColorMapping := TEasyColorMapping(Max(cbColorSpeedSetting.ItemIndex, 0));
      Styles.Assign(FStyles);
      TabStops := cbTabStops.Text;
      KeyMapping := TEasyKeyMapping(Max(cbEditorSpeedSetting.ItemIndex, 0));
      BlockIndent := StrToIntDef(cbBlockIndent.Text, 2);
      Font.Name := cbEditorFont.Text;
      Font.Size := StrToInt(cbSize.Text);
      Margin.Position := StrToIntDef(cbRightMargin.Text, 80);
      Gutter.Width := StrToIntDef(cbGutterWidth.Text, 32);
    finally
      EndUpdate;
    end;
  end;
end;

{-------------------------------------------------}

procedure TCustomizeForm.LoadSettings;
begin
  with EasyEditSettings do
  begin
    cbAutoIndentMode.Checked := srAutoIndent in SourceOptions;
    cbInsertMode.Checked := not EditOverwrite;
    cbUseTabCharacter.Checked := srUseTab in SourceOptions;
    cbSmartTab.Checked := srSmartTab in SourceOptions;
    cbOptimalFill.Checked := srOptimalFill in SourceOptions;
    cbBackspaceUnindents.Checked := srBackUnindents in SourceOptions;
    cbCursorThroughTabs.Checked := srCursorThroughTabs in SourceOptions;
    cbGroupUndo.Checked := srGroupUndo in SourceOptions;
    cbCursorBeyondEOF.Checked := srBeyondEof in SourceOptions;
    cbCursorBeyondEol.Checked := srBeyondEol in SourceOptions;
    cbKeepTrailingBlanks.Checked := srLeaveTrailSpaces in SourceOptions;
    cbPersistentBlocks.Checked := eoPersistentBlocks in Options;
    cbOverwriteBlocks.Checked := eoOverWriteBlocks in Options;
    cbEnableSelection.Checked := eoEnableSelection in Options;
    cbEnableDragging.Checked := eoEnableDragging in Options;
    cbEnableSearchHighlight.Checked := eoEnableSearchHighlight in Options;
    cbDoubleClickLine.Checked := eoDblClickLine in Options;
    cbFindTextAtCursor.Checked := eoFindTextAtCursor in Options;
    cbForceCutAndCopyEnabled.Checked := eoForceCutCopy in Options;
    cbEditorSpeedSetting.ItemIndex := Ord(KeyMapping);
    cbBlockIndent.Text := IntToStr(BlockIndent);
    cbTabStops.Text := TabStops;
    cbEditorFont.Text := Font.Name;
    cbSize.Text := IntToStr(Font.Size);
    chbVisibleRightMargin.Checked := eoPaintMargin in Options;
    chbVisibleGutter.Checked := eoPaintGutter in Options;
    cbRightMargin.Text := IntToStr(Margin.Position);
    cbGutterWidth.Text := IntToStr(Gutter.Width);
    cbColorSpeedSetting.ItemIndex := Ord(ColorMapping);
    FStyles.Assign(Styles);
  end;
end;

{-------------------------------------------------}

end.
