{*****************************************************}
{                                                     }
{ Find dialog                                         }
{                                                     }
{ Copyright(c) 2001 Soford - Software For Developers  }
{ http://www.soford.com                               }
{ e-mail: contact@soford.com                          }
{                                                     }
{*****************************************************}

unit EasySearchDlg;

interface
{$I Easy.inc}
uses
  {$IFNDEF EASY_CLX}
  Windows, Messages, Graphics, Controls, Forms, Dialogs, StdCtrls, ExtCtrls,
  {$ELSE}
  QControls, QForms, QStdCtrls, QExtCtrls,
  {$ENDIF}
  SysUtils, Classes,
  EasyControls, EasyUtils, EasySettings, EasyBox;

type
  TFrmFindDlg = class(TForm)
    lbTextToFind: TLabel;
    plButtons: TPanel;
    btOK: TButton;
    btCancel: TButton;
    btHelp: TButton;
    gbOptions: TGroupBox;
    cbCaseSensitive: TCheckBox;
    cbWholeWord: TCheckBox;
    cbRegularExpr: TCheckBox;
    rgScope: TRadioGroup;
    rgOrigin: TRadioGroup;
    gbDirection: TGroupBox;
    rbForward: TRadioButton;
    rbBackward: TRadioButton;
    cbTextToFind: TEasyHistoryEditor;
    procedure FormCreate(Sender: TObject);
    procedure btOKClick(Sender: TObject);
  private
    { Private declarations }
  public
    procedure LoadSettings; virtual;
    procedure SaveSettings; virtual;
    { Public declarations }
  end;

var
  FrmFindDlg: TFrmFindDlg;

implementation
{$IFNDEF EASY_CLX}
{$R *.DFM}
{$ELSE}
{$R *.xfm}
{$ENDIF}

{--------------------------------------------}

procedure TFrmFindDlg.FormCreate(Sender: TObject);
begin
  cbTextToFind.FileName := AddSlash(GetTempDir) + 'dialogs.ini';
  cbTextToFind.LoadHistory;
  LoadSettings;
end;

{--------------------------------------------}

procedure TFrmFindDlg.LoadSettings;
begin
  with SearchSettings do
  begin
    cbTextToFind.Text := TextToFind;
    cbCaseSensitive.Checked := CaseSensitive;
    cbWholeWord.Checked := WholeWordsOnly;
    cbRegularExpr.Checked := RegularExpr;
    if Direction = srForward then
      rbForward.Checked := true
    else
      rbBackward.Checked := true;
    if Scope = scGlobal then
      rgScope.ItemIndex := 0
    else
      rgScope.ItemIndex := 1;
    if Origin = sgFromCursor then
      rgOrigin.ItemIndex := 0
    else
      rgOrigin.ItemIndex := 1
  end;
end;

{--------------------------------------------}

procedure TFrmFindDlg.btOKClick(Sender: TObject);
begin
  SaveSettings;
end;

{--------------------------------------------}

procedure TFrmFindDlg.SaveSettings;
begin
  with SearchSettings do
  begin
    TextToFind := cbTextToFind.Text;
    CaseSensitive := cbCaseSensitive.Checked;
    WholeWordsOnly := cbWholeWord.Checked;
    RegularExpr := cbRegularExpr.Checked;
    if rbForward.Checked then
      Direction := srForward
    else
      Direction := srBackward;
    if rgScope.ItemIndex = 0 then
      Scope := scGlobal
    else
      Scope := scSelectedText;
    if rgOrigin.ItemIndex = 0 then
      Origin := sgFromCursor
    else
      Origin := sgEntireScope;
  end;
end;

initialization
  GlobalSearchDialogClass := TFrmFindDlg;

end.
